﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows.Forms;
//Cần add reference iDriver.dll
using ATSCADA;
//Cần add reference ATPropertyEditor.dll
using ATPropertyEditor;

namespace ATSCADA.iWinTools
{
    //Specify the Toolbox bimap for iUserControl
    [ToolboxBitmap(typeof(Label))]
    public partial class iLabel : Label
    {
        protected string _TagName;//tagname
        protected string _TaskName;//taskname
        //Đối tượng iDriver chứa cấu trúc Task.Tag của ATSCADA
        protected iDriver _Driver;        

        //Add Tag
        /// <summary>
        /// Select Tag for SCADA control
        /// Cần TypeConverter(typeof(TaskConverter)) để người dùng xổ chọn Task.Tag 
        /// </summary>
        [Description("Select Tag for SCADA control")]
        [TypeConverter(typeof(TaskConverter)), CategoryAttribute("ATSCADA Settings")]
        public string TagName
        {
            get
            {
                return _TaskName + "." + _TagName;
            }
            set
            {
                _TaskName = value.Split('.')[0];
                _TagName = value.Split('.')[1];
            }
        }

        /// <summary>
        /// Select Driver for SCADA control
        /// </summary>
        [Description("Select Driver for SCADA control")]
        [Browsable(true), Category("ATSCADA Settings")]
        public iDriver Driver
        {
            get
            {
                return _Driver;
            }
            set
            {
                _Driver = value;
                try
                {
                    _Driver.ConstructionCompleted -= _Driver_ConstructionCompleted;
                }
                catch { }

                try
                {
                    //Sử dụng sự kiện Driver khởi tạo xong để chạy method _Driver_ConstructionCompleted()
                    _Driver.ConstructionCompleted += _Driver_ConstructionCompleted;
                }
                catch { }
            }
        }
    
        //Method khởi tạo User control iLabel
        void _Driver_ConstructionCompleted()
        {         
            try
            {
                //Attach Driver for Item
                //Get the Tag with _TagName            
                if ((_TagName != null) && (_TaskName != null) && (_Driver != null) && _Driver.Task(_TaskName) != null)
                {
                    //Mục đích để cross thread
                    if (InvokeRequired)
                    {
                        MethodInvoker dowork = new MethodInvoker(delegate
                        {
                            //Get Update Event from tag
                            if (_Driver.Task(_TaskName).Tag(_TagName) != null)
                            {
                                //Nếu tag thay đổi giá trị thì gọi method iUpdateValue()
                                _Driver.Task(_TaskName).Tag(_TagName).TagValueChanged += new TagValueHandler(iUpdateValue);
                                // Gán text của ilabel = giá trị của tag
                                this.Text = _Driver.Task(_TaskName).Tag(_TagName).Value;
                            }
                        });
                        //thủ tục cross thread
                        Invoke(dowork);
                        return; 
                    }
                }
            }
            catch { }            
        }

        //Method cập nhật giá trị cho iLabel mỗi khi tag thay đổi giá trị
        private void iUpdateValue(object o, TagValueEventArgs e)
        {
            try
            {
                //Update Label Value                
                string[] nv;
                //Bắt dữ liệu thay đổi từ sự kiện TagValueChanged của Tag. Dữ liệu này có format Tashname/TagName/Value
                //Băm ra thành 3 phần tử mảng
                nv = e.NewDisplayValue.Split('/');
                
                if ((nv[0] == _TaskName) && (nv[1] == _TagName))
                {
                    //Mục đích để cross thread
                    if (InvokeRequired)
                    {
                        MethodInvoker method = new MethodInvoker(delegate { this.Text = nv[2]; });

                        Invoke(method);
                        return;
                    }
                }

            }
            catch (Exception ex)
            {
                throw new Exception(ex.ToString());
            }
        }

    }
}
